#
#++
#
#  Copyright Digital Equipment Corporation 1996. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#
#++
#
#  MODULE:
#
#      D_ARP.DDL
#
#  ABSTRACT:
#
#	DDL program to DECODE ARP packets.
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      12 December 1996
#
#  MODIFICATION HISTORY:
#
#
INCLUDE ddlinc.ddl

# CONSTANTS
#
CONSTANT ARP_hardware_type = 0,
         ARP_protocol_type = 2,
         ARP_hardware_address_length = 4,
         ARP_protocol_address_length = 5,
         ARP_operation = 6,
         ARP_rest = 8,
         ARP_sizeof = 9;


# Ethernet ARP CONSTANTs
#
CONSTANT ARP_ether_sha = 8,
         ARP_ether_spa = 14,
         ARP_ether_tha = 18,
         ARP_ether_tpa = 24;

# DECLAREs
#
DECLARE PST pst_arp, pst_arp_hardware;
DECLARE DECODE d_arp_req,d_arp_resp,d_arp_rreq,d_arp_rresp,
        d_arp_dreq,d_arp_dresp,d_arp_derr,d_arp_ireq,
        d_arp_iresp;


# NAME LISTS
# ARP Hardware Types
#
NAME_LIST arpHardwareTypes {
    TITLE = "Hardware type",
    OUT_OF_RANGE = "illegal" } =

       "",
       "Ethernet (10Mb)",                      # 1 
       "Experimental Ethernet (3Mb)",          # 2 
       "Amateur Radio AX.25",                  # 3 
       "Proteon ProNET Token Ring",            # 4 
       "Chaos",                                # 5 
       "IEEE 802 Networks",                    # 6 
       "ARCNET",                               # 7 
       "Hyperchannel",                         # 8 
       "Lanstar",                              # 9 
       "Autonet Short Address",                # 10 
       "LocalTalk",                            # 11 
       "LocalNet (IBM PCNet or SYTEK LocalNET)",# 12 
       "Ultra link",                           # 13 
       "SMDS",                                 # 14 
       "Frame Relay",                          # 15 
       "Asynchronous Transmission Mode";       # 16 

# ARP Operation Types
#
NAME_LIST arpOperationTypes {
    TITLE = "Operation type",
    OUT_OF_RANGE = "illegal" } =

    "",
    "ARP request",                  # 1 
    "ARP response",                 # 2 
    "Reverse ARP request",          # 3 
    "Reverse ARP response",         # 4 
    "Dynamic Reverse ARP request",  # 5 
    "Dynamic reverse ARP response", # 6 
    "Dynamic reverse ARP error",    # 7 
    "Inverse ARP request",          # 8 
    "Inverse ARP response"          # 9 
;


# PST: pst_arp
# Entry point PST, SWITCHes on the ARP operation type.
#
PST pst_arp( word(ARP_operation) ) {TYPE = "ARP"} = 

        d_arp_req,
        d_arp_resp,
        d_arp_rreq,
        d_arp_rresp,
        d_arp_dreq,
        d_arp_dresp,
        d_arp_derr,
        d_arp_ireq,
        d_arp_iresp;

DECODE d_arp_req(1) {            # RFC826
    NEXT = pst_arp_hardware,
    TITLE = "ARP request",
    BRIEF_TITLE = "ARP Req",
    PLEVEL = HL_ARP } = 

    list( byte(ARP_operation + 1), 0, arpOperationTypes ),
    list( byte(ARP_hardware_type + 1), 0, arpHardwareTypes ),
    hex( word( ARP_protocol_type ), 4, "Protocol type" ),
    decimal( byte(ARP_hardware_address_length), 0, "Hardware address length" ),
    decimal( byte(ARP_protocol_address_length), 0, "Protocol address length" )
    ;

DECODE d_arp_resp(2) {
    NEXT = pst_arp_hardware,
    TITLE = "ARP response",
    BRIEF_TITLE = "ARP Resp",
    PLEVEL = HL_ARP } = REF d_arp_req;

DECODE d_arp_rreq(3) {
    NEXT = pst_arp_hardware,
    TITLE = "Reverse ARP request",
    BRIEF_TITLE = "ARP RevRqst",
    PLEVEL = HL_ARP }  = REF d_arp_req;

DECODE d_arp_rresp(4) {              # RFC903
    NEXT = pst_arp_hardware,
    TITLE = "Reverse ARP response",
    BRIEF_TITLE = "ARP RevResp",
    PLEVEL = HL_ARP } = REF d_arp_req;

DECODE d_arp_dreq(5) {
    NEXT = pst_arp_hardware,
    TITLE = "Dynamic Reverse ARP request",
    BRIEF_TITLE = "ARP DynRqst",
    PLEVEL = HL_ARP } = REF d_arp_req;

DECODE d_arp_dresp(6) {
    NEXT = pst_arp_hardware,
    TITLE = "Dynamic Reverse ARP response",
    BRIEF_TITLE = "ARP DynResp",
    PLEVEL = HL_ARP } = REF d_arp_req;

DECODE d_arp_derr(7) {
    NEXT = pst_arp_hardware,
    TITLE = "Dynamic Reverse ARP error",
    BRIEF_TITLE = "ARP DynErr ",
    PLEVEL = HL_ARP } = REF d_arp_req;

DECODE d_arp_ireq(8) {               # RFC1293
    NEXT = pst_arp_hardware,
    TITLE = "Inverse ARP request",
    BRIEF_TITLE = "ARP InvRqst",
    PLEVEL = HL_ARP } = REF d_arp_req;

DECODE d_arp_iresp(9) {
    NEXT = pst_arp_hardware,
    TITLE = "Inverse ARP response",
    BRIEF_TITLE = "ARP InvResp",
    PLEVEL = HL_ARP } = REF d_arp_req;



# PST: pst_arp_hardware
# SWITCH on the hardware type.  
#
DECLARE DECODE d_arp_ether, d_arp_fr, d_arp_hw_default;
PST pst_arp_hardware( byte(ARP_hardware_type+1) ) {
    DEFAULT = d_arp_hw_default
    } =

    d_arp_ether,
    d_arp_fr
;


# DECODE: d_arp_ether
# DECODE the ethernet specific parts of the ARP packet.
#
DECLARE PST pst_arp_protocol;
DECODE d_arp_ether(1) {
    NEXT = pst_arp_protocol } =
	
	mac_name( ARP_ether_sha, 0, "Sender MAC address" ) {BRIEF},
        mac_name( ARP_ether_tha, 0, "Target MAC address" );


# DECODE: d_arp_fr
# DECODE the Frame Relay specific parts of the ARP packet.
#
DECODE d_arp_fr(15) {
    NEXT = pst_arp_protocol } =
        $1 = byte(ARP_hardware_address_length),     # hardware address length 
        $2 = byte(ARP_protocol_address_length), 
        $2 = $2 + ARP_rest + $1,                    # start of target hw address 
	fr_dlci( ARP_rest, $1, "Sender FR address" ) {BRIEF},
    	fr_dlci( $2, $1, "Target FR address" );


# DECODE d_arp_hw_default
# DECODE an unknown hardware type.
#
DECODE d_arp_hw_default(0) {
    NEXT = pst_arp_protocol } =

        $1 = byte(ARP_hardware_address_length),     # hardware address length 
        $2 = byte(ARP_protocol_address_length), 
        $2 = $2 + ARP_rest + $1,                    # start of target hw address 
	dump_hex_bytes(ARP_rest,$1,"Sender Hardware address") {BRIEF},
        dump_hex_bytes($2,$1,"Target Hardware address")
        ;


# PST: pst_arp_protocol
# SWITCH on the protocol type.  
#
DECLARE DECODE d_arp_ip, d_arp_pr_default;
PST pst_arp_protocol( word(ARP_protocol_type) ) {
    DEFAULT = d_arp_pr_default
    } =

    d_arp_ip
;


# DECODE: d_arp_ip
# DECODE the IP specific parts of the ARP packet.
#
DECODE d_arp_ip(0x800) {
    CONSUME = remaining() } =

	ip_address( long(ARP_ether_spa), 0,"Sender IP address") {BRIEF},
	ip_address( long(ARP_ether_tpa), 0,"Target IP address") {BRIEF};


# DECODE d_arp_pr_default
# DECODE an unknown protocol type.
#
DECODE d_arp_pr_default(0) {
    CONSUME = remaining()
    } =

        $1 = byte(ARP_hardware_address_length),     # hardware address length 
        $2 = byte(ARP_protocol_address_length),     # protocol address length 
        $3 = $1 + ARP_rest,                         # start of spa 
        $4 = $3 + $2 + $1,                          # start of tpa 
	dump_hex_bytes($3,$2,"Sender Protocol address") {BRIEF},
	dump_hex_bytes($4,$2,"Target Protocol address") {BRIEF}
        ;


