#
#++
#
#  Copyright Digital Equipment Corporation 1996. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#**++
#
#  MODULE:
#
#      D_ENET.DDL
#
#  ABSTRACT:
#
#      DDL program to DECODE CSMA-CD packets (Ethernet and 802.3).
#      The entry point PST is PST_csmacd which requires the current offset to
#      be positioned at the start of the MAC header.
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      12 December 1996
#
#  MODIFICATION HISTORY:
#
INCLUDE ddlinc.ddl

# Ethernet CONSTANTs
#
CONSTANT ENET_maxlen = 1500;

# Ethernet Datalink Packet offsets
#
CONSTANT ENET_da = 0,
	 ENET_sa = 6;

# 802.3 style
#
CONSTANT ENET_len = 12,
         ENET_8023_size = 14,
         ENET_sap = 14;

CONSTANT ENET_llc2_control = 2;		# Offset from the _sap field

# EthernetII style
#
CONSTANT ENET_prot = 12,
         ENET_user_length = 2;          # Offset from the _prot field

# Configuration Testing Protocol
#
CONSTANT CTP_function = 0,
         CTP_receipt = 2,               # function=1
         CTP_forward = 2,               # function=2
         CTP_next = 8;                  # function=2

CONSTANT
	DECNB_version = 2,
	DECNB_service = DECNB_version+2,
	DECNB_message = DECNB_service+2,
	DECNB_code    = DECNB_message+1,
	DECNB_source  = DECNB_code+1,
	DECNB_dest    = DECNB_source+16,
	DECNB_s_stat  = DECNB_dest+16,
	DECNB_d_stat  = DECNB_s_stat+1,
	DECNB_length  = DECNB_d_stat+1,
	DECNB_object  = DECNB_length+2,
	DECNB_sizeof  = DECNB_object+1;
	
	
# Declarations
#
DECLARE DECODE d_ethernet, d_8023, d_ctp_term, d_ctp_forw,
               d_ethernet_last, d_ethernet_sca,
	       d_ethernet_mop, d_ethernet_moprc,
	       d_ethernet_piv, d_ethernet_ctp,
	       d_8022_snap, d_ethernet_netbeui, d_netbeui_i_frame, d_netbeui_rr,
	       d_snap_encap, d_snap_bridge,
	       d_snap_apple, d_snap_phasev, d_apple_8023, d_apple_arp_8023,
	       d_dna_8023_mop, d_dna_8023_moprc, d_dna_8023_lat, d_dna_8023_ctp,
	       d_iana_8024c, d_iana_8024, d_iana_8025c, d_iana_8025, d_iana_8026,
	       d_snap_iana,;

DECLARE NAME_TABLE EthernetProtocols;
DECLARE PST pst_ethernet, pst_8022, pst_8022_snap, pst_arp, pst_apple_8023, pst_dna_8023;
DECLARE PST pst_ip, lat_types, atlk_types, aarp_types,
	    ipx_types, ipv6_types, ctp_types, mop_types, piv_pad,
	    bridge_types, llc2_packets, ppp_protocols, netbeui_datagram,
	    osi_types, pst_fddi, pst_iana_8023;


# Name Lists
# CTP Functions
NAME_LIST ctp_functions {
    TITLE = "Next function",
    OUT_OF_RANGE = "illegal",
    } =
        "Reply",                         # 1
        "Forward",                       # 2
	;
NAME_LIST decnb_service_classes {
    TITLE = "Service Class",
    OUT_OF_RANGE = "Not Decoded",
    } =
	"Unknown",			# 0
        "Datagram",			# 1
        "Adapter Claim"			# 2
	;

# Name Table
# EthernetProtocols
NAME_TABLE EthernetProtocols {
    TITLE = "Protocol type",
    };


# CSMACD Packets PST
# This is the primary PST for CSMA-CD packets,  this PST SWITCHes
# between Ethernet and 802.3 packets.  The current offset is expected
# to be at the start of the ethernet MAC header.  The word at the
# ENET_len offset is used to distinguish between the two.
#
PST pst_csmacd (uword_gt( ENET_maxlen, ENET_len )) {
    TITLE = "NI length",
    TYPE = "CSMA-CD" } = 
	d_ethernet,
	d_8023,
	;


# d_ethernet DECODE
# DECODEs the Ethernet header and passes control to the pst_ethernet PST.
#
DECODE d_ethernet(1) {
    NEXT = pst_ethernet,
    TITLE = "EthernetII",
    BRIEF_TITLE = "Ether",
    PLEVEL = HL_ENET,
    CONSUME = ENET_len, } =

        mac_name(  ENET_da,  0, "Destination Address" ) {BRIEF},
        mac_name(  ENET_sa,  0, "Source Address" ) {BRIEF},
	name(  "Protocol type", EthernetProtocols, le_word(ENET_prot) ),
	;


# d_8023 DECODE
# DECODEs the 802.3 header and passes control to the ...
#
DECODE d_8023(0) {
    NEXT = pst_8022,
    TITLE = "IEEE 802.3", 
    BRIEF_TITLE = "802.3",
    PLEVEL = HL_I8023,
    CONSUME = ENET_8023_size } =

	mac_name(  ENET_da,  0, "Destination Address" ) {BRIEF},
	mac_name(  ENET_sa,  0, "Source Address" ) {BRIEF},
	length(  uword(ENET_len),  ENET_8023_size, "length" ) {BRIEF},
	hex(  byte(ENET_sap),  2, "destination SAP" )
	;

DECODE d_ethernet_pcsa(0x8040) {
    NEXT = smb_decode,
    TITLE = "DEC NETBios", 
    BRIEF_TITLE = "DECNB",
    PLEVEL = HL_I8023,
    CONSUME = DECNB_sizeof
    } =
	print( "Version", "V%d.%d", byte (DECNB_version),byte(DECNB_version+1)),
	list ( byte ( DECNB_service ), 0, decnb_service_classes ),
	decimal ( byte ( DECNB_message ) , 0, "Message function"),
	decimal ( byte ( DECNB_code ) , 0, "Message ID"),
	string_dump_n_bytes ( DECNB_source, 16, "Source Adapter"),
	string_dump_n_bytes ( DECNB_dest, 16, "Destination Adapter"),
	decimal (byte (DECNB_s_stat), 0, "Source status"),
	decimal (byte (DECNB_d_stat), 0, "Destination status"),
	decimal (byte (DECNB_object), 0, "DECnet object"),
	length( le_uword ( DECNB_length ) , DECNB_sizeof , "length" ) {BRIEF}
	;


# pst_ethernet PST
# SWITCH on the protocol type in the Ethernet packet, the current
# position must be pointing to the first byte after the 12 byte
# header (i.e. the protocol type).
#
PST pst_ethernet( word(0) ) {
    TITLE = "Enet protocol" } = 
        0x0806 -> pst_arp { CONSUME = 2, TITLE = "ARP", PLEVEL = HL_ARP },
        0x0800 -> pst_ip { CONSUME = 2, TITLE = "IP", PLEVEL = 0 },
        0x6004 -> lat_types { CONSUME = 2, TITLE = "LAT", PLEVEL = HL_LAT },
        0x809B -> atlk_types { CONSUME = 2, TITLE = "AppleTalk", PLEVEL = HL_ATLK },
        0x80F3 -> aarp_types { CONSUME = 2, TITLE = "Appletalk ARP", PLEVEL = 0 },
        0x8137 -> ipx_types { CONSUME = 2, TITLE = "IPX - ethernet-II encapsulation", PLEVEL = HL_IPX , BRIEF_TITLE = "IPX"},
        0x86DD -> ipv6_types { CONSUME = 2, TITLE = "IPV6", PLEVEL = HL_IP },
	0x880B -> ppp_protocols { TITLE = "PPP", PLEVEL = HL_PPP },
	0x8021 -> ppp_protocols { TITLE = "PPP", PLEVEL = HL_PPP },
	0xC021 -> ppp_protocols { TITLE = "PPP", PLEVEL = HL_PPP },
	0xC223 -> ppp_protocols { TITLE = "PPP", PLEVEL = HL_PPP },
	0x80FD -> ppp_protocols { TITLE = "PPP", PLEVEL = HL_PPP },
	d_ethernet_piv,
	d_ethernet_sca,
	d_ethernet_last,
	d_ethernet_ctp,
	d_ethernet_mop,
	d_ethernet_moprc,
	d_ethernet_pcsa
	;


# pst_8022 PST
# SWITCH on the destination SAP in the 802.3 packet, the current
# position must be pointing to the DSAP.
#
PST pst_8022( byte( 0 ) ) {
    TITLE = "IEEE802.2 SAP",
    TYPE = "SAPS" } = 
        0x42 -> bridge_types { CONSUME = 3 },
        0x7E -> llc2_packets,
	d_8022_snap,
        0xE0 -> ipx_types { CONSUME = 3 , TITLE = "IPX 802.2"},
	d_ethernet_netbeui,
        0xFE -> osi_types { CONSUME = 3 },
        0xFF -> ipx_types,
	;

PST pst_netbeui( l2_type( ENET_llc2_control ) ) {
    TITLE = "LLC2-Netbeui-Frame type"
    } = 
        d_netbeui_i_frame,
        d_netbeui_rr,
        0x03 -> netbeui_datagram { CONSUME = 3 },
        0x0f -> netbeui_datagram { CONSUME = 3 , TITLE = "Disconnect Mode", BRIEF_TITLE = "DM", PLEVEL = HL_ENET},
        0x43 -> netbeui_datagram { CONSUME = 3 , TITLE = "Disconnect", BRIEF_TITLE = "DISC", PLEVEL = HL_ENET},
        0x63 -> netbeui_datagram { CONSUME = 3 , TITLE = "Unnumberred Acknowledge", BRIEF_TITLE = "UA", PLEVEL = HL_ENET},
        0x6f -> netbeui_datagram { CONSUME = 3 , TITLE = "SABME", PLEVEL = HL_ENET}
	;
DECODE d_netbeui_i_frame(0) {
    NEXT = netbeui_datagram,
    TITLE = "I frame",
    CONSUME = 4,
    PLEVEL = HL_ENET,
    BRIEF_TITLE = " I "}=
	llc2_seqno(ENET_llc2_control,0,""){BRIEF}
	;
DECODE d_netbeui_rr(1) {
    TITLE = "Receiver Ready",
    CONSUME = 4,
    PLEVEL = HL_ENET,
    BRIEF_TITLE = "RR "}=
	llc2_seqno(ENET_llc2_control,0,""){BRIEF}
	;

DECODE d_ethernet_netbeui(0xF0) {
    NEXT = pst_netbeui,
    TITLE = "LLC2 - Netbeui",
    PLEVEL = HL_ENET,
    BRIEF_TITLE = "NETBEUI"}=;

DECODE d_ethernet_sca(0x6007) {
    TITLE = "SCA", 
    PLEVEL = HL_ZERO,
    CONSUME = 2} =;

DECODE d_ethernet_last(0x8041) {
    TITLE = "LAST", 
    PLEVEL = HL_UNKNOWN,
    CONSUME = 2 } =;

DECODE d_ethernet_ctp(0x9000) {
    NEXT = ctp_types,
    TITLE = "Configuration Testing Protocol",
    CONSUME = le_word_plus( 4, 2 ),
    PLEVEL = HL_CTP,
    BRIEF_TITLE = "CTP"}=;

DECODE d_ethernet_mop(0x6001) {
    NEXT = mop_types,
    TITLE = "MOP dump/load",
    CONSUME = 4,
    PLEVEL = HL_MOP,
    BRIEF_TITLE = "MOP-DL" } = 

	length( le_uword(ENET_user_length),  ENET_user_length + 2, "Length")  {BRIEF}
	;

DECODE d_ethernet_moprc(0x6002) {
    NEXT = mop_types,
    TITLE = "MOP console",
    CONSUME = 4,
    PLEVEL = HL_MOP,
    BRIEF_TITLE = "MOP-RC" } = 

	length( le_uword(ENET_user_length),  ENET_user_length + 2, "Length")  {BRIEF}
	;

DECODE d_ethernet_piv(0x6003) {
    NEXT = piv_pad,
    TITLE = "PIV",
    PLEVEL = HL_PIV,
    CONSUME = 4 } = 

	length( le_uword(ENET_user_length),  ENET_user_length + 2, "Length")  {BRIEF}
	;


# 802.2 DECODEs
# The following are DECODEs for the 802.2 SAPs
#
DECODE d_8022_snap(0xAA) {
    NEXT = pst_8022_snap,
    TITLE = "SNAP",
    PLEVEL = HL_I8022,
    CONSUME = 3 } =

	dump_hex_bytes( 3,  5, "PID" )
	;

# SNAP PST
# The current position must be on the OUI field.  The SWITCH occurs on
# the 3 byte OUI.
#
PST pst_8022_snap( b3yte(0) ) {
    TITLE = "IEEE802.2E SNAP SAP",
    TYPE = "SNAPS"
    } =
    d_snap_encap,
    d_snap_bridge,
    d_snap_apple,
    d_snap_phasev,
    d_snap_iana
    ;


# DECODEs for 802.2 SNAP
#
DECODE d_snap_encap(0) {
    TITLE = "Encapsulated Ethernet",
    BRIEF_TITLE = "ENCAP",
    NEXT = pst_ethernet,
    PLEVEL = HL_ZERO,
    CONSUME = 3
    } =;

DECODE d_snap_bridge(0xF8) {
    TITLE = "Bridge OUI 000000",
    BRIEF_TITLE = "BRDG",
    NEXT = pst_ethernet,
    PLEVEL = HL_ZERO,
    CONSUME = 3
    } =;

DECODE d_snap_apple(0x80007) {
    TITLE = "Apple",
    BRIEF_TITLE = "APL",
    NEXT = pst_apple_8023,
    PLEVEL = HL_ZERO,
    CONSUME = 3
    } =;

DECODE d_snap_phasev(0x8002B) {
    TITLE = "DNA phase V protocol",
    BRIEF_TITLE = "PhaseV",
    NEXT = pst_dna_8023,
    PLEVEL = HL_ZERO,
    CONSUME = 3
    } =;

DECODE d_snap_iana(0x0080C2) {
    TITLE = "IANA",
    BRIEF_TITLE = "ietf",
    NEXT = pst_iana_8023,
    PLEVEL = HL_ZERO,
    CONSUME = 3
    } =;


# 802.3 AppleTalk PST and DECODEs
#
PST pst_apple_8023( word(0) ) {
    TITLE = "Apple Protocol" } =

    d_apple_8023,
    d_apple_arp_8023
    ;

DECODE d_apple_8023(0x809B) {
    TITLE = "Appletalk",
    BRIEF_TITLE = "AplTk",
    NEXT = atlk_types,
    PLEVEL = HL_ZERO,
    CONSUME = 2
    } =;

DECODE d_apple_arp_8023(0x80F3) {
    TITLE = "Appletalk ARP",
    BRIEF_TITLE = "AplTkArp",
    NEXT = aarp_types,
    PLEVEL = HL_ZERO,
    CONSUME = 2
    } =;


# DNA PST and DECODEs
#
PST pst_dna_8023( word(0) ) {
    TITLE = "DNA protocol" } =
    d_dna_8023_mop,
    d_dna_8023_moprc,
    d_dna_8023_lat,
    d_dna_8023_ctp
    ;

DECODE d_dna_8023_mop(0x6001) {
    NEXT = mop_types,
    TITLE = "MOP dump/load",
    BRIEF_TITLE = "MOP D/L",
    PLEVEL = HL_MOP,
    CONSUME = 2
    } =;

DECODE d_dna_8023_moprc(0x6002) {
    NEXT = mop_types,
    TITLE = "MOP console",
    BRIEF_TITLE = "MOP RC",
    PLEVEL = HL_MOP,
    CONSUME = 2
    } =;

DECODE d_dna_8023_lat(0x6004) {
    NEXT = lat_types,
    TITLE = "LAT",
    PLEVEL = HL_LAT,
    CONSUME = 2
    } =;

DECODE d_dna_8023_ctp(0x9000) {
    NEXT = ctp_types,
    TITLE = "Configuration Testing Protocol",
    BRIEF_TITLE = "CTP",
    PLEVEL = HL_CTP,
    CONSUME = 2
    } =;



# CTP PST
#
PST pst_ctp( le_word(CTP_function) ) {
    TITLE = "CTP function",
    TYPE = "CTP" } =
    
	d_ctp_term,
	d_ctp_forw
	;


# CTP Terminating Data DECODE
#
DECODE d_ctp_term(1) {
    TITLE = "terminating data",
    PLEVEL = HL_ENET,
    CONSUME = 4
    } =

	decimal( le_word(CTP_receipt), 0,"receipt number" )
	;

# CTP Forwarding Data DECODE
#
DECODE d_ctp_forw(2) {
    TITLE = "forwarding data",
    PLEVEL = HL_ENET,
    CONSUME = 8
    } =

	mac_name( CTP_forward, 0,"destination address" ),
	list( byte(CTP_next), 0,ctp_functions )
	;


#
# IANA PST and DECODEs
#
PST pst_iana_8023( word(0) ) {
    TITLE = "IANA bridge encapsulation"
    } =
	0x0001 -> pst_csmacd { TITLE = "ethernet with CRC", CONSUME = 2, PLEVEL = HL_ENET },
	0x0004 -> pst_fddi { TITLE = "FDDI with CRC", CONSUME = 3, PLEVEL = HL_FDDI },
	0x0007 -> pst_csmacd { TITLE = "ethernet", CONSUME = 2, PLEVEL = HL_ENET },
	0x000A -> pst_fddi { TITLE = "FDDI", CONSUME = 3, PLEVEL = HL_FDDI },
	0x000E -> bridge_types { TITLE = "Bridge Configuration", CONSUME = 2, PLEVEL = HL_BRIDGE, BRIEF_TITLE = "802.1d"},
	d_iana_8024c,
	d_iana_8025c,
	d_iana_8024,
	d_iana_8025,
	d_iana_8026
    ;

DECODE d_iana_8024c (0x0002) {
    TITLE = "IEEE 802.4 with CRC" ,
    CONSUME = 2,
    PLEVEL = HL_ZERO,
    BRIEF_TITLE = "802.4" } = ;

DECODE d_iana_8025c (0x0003) {
    TITLE = "IEEE 802.5 with CRC" ,
    CONSUME = 2,
    PLEVEL = HL_ZERO,
    BRIEF_TITLE = "802.5" } = ;

DECODE d_iana_8024 (0x0008) {
    TITLE = "IEEE 802.4"          ,
    CONSUME = 2,
    PLEVEL = HL_ZERO,
    BRIEF_TITLE = "802.4" } = ;

DECODE d_iana_8025 (0x0009) {
    TITLE = "IEEE 802.5"          ,
    CONSUME = 2,
    PLEVEL = HL_ZERO,
    BRIEF_TITLE = "802.5" } = ;

DECODE d_iana_8026 (0x000B) {
    TITLE = "IEEE 802.6"          ,
    CONSUME = 2,
    PLEVEL = HL_ZERO  ,
    BRIEF_TITLE = "802.6" } = ;


#*****************************************************************
#                       CRC Included Version                     *
#*****************************************************************

DECLARE DECODE d_ethernet_crc, d_8023_crc;

# CSMACD Packets PST (with CRC appended)
# As pst_csmacd but the packets have the ethernet CRC appended.
#
PST pst_csmacd_crc( uword_gt( ENET_maxlen, ENET_len ) ) {
    TITLE = "NI length",
    TYPE = "CSMA-crc" } = 
	d_ethernet_crc,
	d_8023_crc
	;

# d_ethernet_crc DECODE
# DECODEs the Ethernet header and passes control to the pst_ethernet PST.
#
DECODE d_ethernet_crc(1) {
    NEXT = pst_ethernet,
    TITLE = "EthernetII",
    BRIEF_TITLE = "Ether",
    PLEVEL = HL_ENET,
    CONSUME = ENET_len } =

	mac_name(  ENET_da,  0, "Destination Address" ) {BRIEF},
	mac_name(  ENET_sa,  0, "Source Address" ) {BRIEF},
	name(  "Protocol type", EthernetProtocols, le_word(ENET_prot) ),
	ethernet_crc(  0 )
	;

# d_8023 DECODE
# DECODEs the 802.3 header and passes control to the ...
#
DECODE d_8023_crc(0) {
    NEXT = pst_8022,
    TITLE = "IEEE 802.3", 
    BRIEF_TITLE = "802.3",
    PLEVEL = HL_I8023,
    CONSUME = ENET_8023_size } =

	mac_name(  ENET_da,  0, "Destination Address" ) {BRIEF},
	mac_name(  ENET_sa,  0, "Source Address" ) {BRIEF},
	ethernet_crc( 0 ),
	length(  uword(ENET_len),  ENET_8023_size, "length" ) {BRIEF},
	hex(  byte(ENET_sap),  2, "destination SAP" )
	;



