#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_IGMP.DDL
#
#  ABSTRACT:
#
#	DDL program to DECODE IGMP packets.
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      14 Jun 1997
#
#  MODIFICATION HISTORY:
#
#	23-Jul-97	A A Hart
#	Support IGMP V2 and V1 messages.
#
#
#
INCLUDE ddlinc.ddl

#
# IGMP Packet Offsets
#
CONSTANT
	IGMP_vertyp		= 0,		# V2 combines the V1 version and type fields
	IGMP_code		= 1,
	IGMP_maxresp		= 1,
	IGMP_checksum		= 2,
	IGMP_group_address	= 4,
	IGMP_sizeof		= 8;

DECLARE PST default_igmp_types, pst_ip_protocols;



# 
# DECODE: d_igmp2_report
# IGMP Version 2 host membership report.
#
DECODE d_igmp2_report( 0x16 ) {
	TITLE = "Version 2 Membership Report",
	BRIEF_TITLE = "IGMP MemberReport",
	PLEVEL = HL_IGMP,
	CONSUME = remaining()
	} =

	    hex( byte( IGMP_vertyp ), 0, "Type" ),
	    decimal( byte( IGMP_maxresp ), 0, "Max Response Time (tenths)" ),
	    ip_checksum( IGMP_checksum, (IGMP_sizeof << 16) + IGMP_checksum, "Verified checksum" ),
	    ip_address( long( IGMP_group_address ), TRANSLATE, "Group address" )
	    ;


# 
# DECODE: d_igmp2_leave
# IGMP Version 2 host membership leave.
#
DECODE d_igmp2_leave( 0x17 ) {
	TITLE = "Leave Group",
	BRIEF_TITLE = "IGMP Leave",
	PLEVEL = HL_IGMP,
	CONSUME = remaining()
	} =

		REF d_igmp2_report;



# 
# DECODE: d_igmp1_report
# IGMP Version 1 host membership report.
#
DECODE d_igmp1_report( 0x12 ) {
	TITLE = "Version 1 Membership Report",
	BRIEF_TITLE = "IGMP MemberReport",
	PLEVEL = HL_IGMP,
	CONSUME = remaining()
	} =

		REF d_igmp2_report;


# 
# DECODE: d_igmp_query
# IGMP Version 1 (and 2) host membership query.
#
DECODE d_igmp_query( 0x11 ) {
	TITLE = "Membership Query",
	BRIEF_TITLE = "IGMP MemberQuery",
	PLEVEL = HL_IGMP,
	CONSUME = remaining()
	} =
		REF d_igmp2_report;


#
# PST: pst_igmp
# Switch on the IGMP type field, use the builtin IGMP decode
# if we don't get a match here.
#
DECODE d_igmp_unknown(0) { NEXT = default_igmp_types } = ;
PST pst_igmp( byte( IGMP_vertyp ) ) {
	TITLE = "IGMP",
	DEFAULT = d_igmp_unknown,
	TYPE = "IGMP"
	} =
		d_igmp2_report,
		d_igmp_query,
		d_igmp2_leave,
		d_igmp1_report,
		;

#
# DECODE: d_igmp
# Main entry point for IGMP packets.
# CO = IP header
# $32 = IP header length
#
DECODE d_igmp( 2 ) {
	ATTACH = pst_ip_protocols,
	NEXT = pst_igmp,
	CONSUME = $32
	} =
		;
