#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_OSPF.DDL
#
#  ABSTRACT:
#
#	DDL program to DECODE IP V4 OSPF packets.
#	Currently doesn't support TOS fields (these are going to be removed from
#	the OSPF standard anyway).
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      24 Mar 1997
#
#  MODIFICATION HISTORY:
#
#  002	4-Nov-1997	A A Hart
#    o	Validate the checksum in OSPF LSAs.
#
#  001	3-Jun-1997	A A Hart
#    o	Add TYPE optional parameter to the pst_ospf_types clause to
#       include it in the command line and link it to the events file.
#    o  Support MOSPF.
#
#    This file can now be dynamically loaded.
#
INCLUDE ddlinc.ddl

# CONSTANTs
#
CONSTANT IP_sizeof 		      = 20;

CONSTANT 
	OSPF_version 		      = 0,
	OSPF_type 		      = 1,
	OSPF_packet_length 	      = 2,
	OSPF_router_id 	       	      = 4,
	OSPF_area_id 		      = 8,
	OSPF_checksum 		      = 12,
	OSPF_autype 		      = 14,
	OSPF_authentication 	      = 16,
	OSPF_sizeof 		      = 24;

CONSTANT
	OSPF_HELLO_network_mask       = 0 + OSPF_sizeof,
	OSPF_HELLO_hello_int          = 4 + OSPF_sizeof,
	OSPF_HELLO_options            = 6 + OSPF_sizeof,
	OSPF_HELLO_router_priority    = 7 + OSPF_sizeof,
	OSPF_HELLO_dead_interval      = 8 + OSPF_sizeof,
	OSPF_HELLO_designated_router  = 12 + OSPF_sizeof,
	OSPF_HELLO_backup_router      = 16 + OSPF_sizeof,
	OSPF_HELLO_neighbor           = 20 + OSPF_sizeof,
	OSPF_HELLO_sizeof             = 20 + OSPF_sizeof;

CONSTANT
	OSPF_DD_options		      = 2 + OSPF_sizeof,
	OSPF_DD_ims		      = 3 + OSPF_sizeof,
	OSPF_DD_sequence_no	      = 4 + OSPF_sizeof,
	OSPF_DD_sizeof		      = 8 + OSPF_sizeof;

CONSTANT
	OSPF_LSH_age		      = 0,
	OSPF_LSH_options	      = 2,
	OSPF_LSH_type		      = 3,
	OSPF_LSH_id		      = 4,
	OSPF_LSH_adv_router	      = 8,
	OSPF_LSH_sequence_no	      = 12,
	OSPF_LSH_checksum	      = 16,
	OSPF_LSH_length		      = 18,
	OSPF_LSH_sizeof		      = 20;

CONSTANT
	OSPF_LSREQ_type		      = 0,
	OSPF_LSREQ_id		      = 4,
	OSPF_LSREQ_adv_router	      = 8,
	OSPF_LSREQ_sizeof	      = 12;

CONSTANT
	OSPF_LSUP_number	      = 0 + OSPF_sizeof,
	OSPF_LSUP_sizeof	      = 4 + OSPF_sizeof;

CONSTANT
	OSPF_RTR_veb		      = 0 + OSPF_LSH_sizeof,
	OSPF_RTR_links		      = 2 + OSPF_LSH_sizeof,
	OSPF_RTR_id		      = 0,
	OSPF_RTR_data		      = 4,
	OSPF_RTR_type		      = 8,
	OSPF_RTR_ntos		      = 9,
	OSPF_RTR_metric0	      = 10,
	OSPF_RTR_sizeof		      = 12;

CONSTANT
	OSPF_NET_mask		      = 0 + OSPF_LSH_sizeof,
	OSPF_NET_sizeof		      = 4 + OSPF_LSH_sizeof;

CONSTANT
	OSPF_SUM_mask		      = 0 + OSPF_LSH_sizeof,
	OSPF_SUM_tos_metric	      = 4 + OSPF_LSH_sizeof;

CONSTANT
	OSPF_EXT_mask		      = 0 + OSPF_LSH_sizeof,
	OSPF_EXT_tos_metric	      = 4 + 0 + OSPF_LSH_sizeof,
	OSPF_EXT_fwd_addr	      = 4 + 4 + OSPF_LSH_sizeof,
	OSPF_EXT_ext_tag	      = 4 + 8 + OSPF_LSH_sizeof,
	OSPF_EXT_sizeof		      = 4 + 12 + OSPF_LSH_sizeof;

CONSTANT
	OSPF_GM_vertex_type	      = 0 + OSPF_LSH_sizeof,
	OSPF_GM_vertex_id	      = 4 + OSPF_LSH_sizeof;



#
# Authentication Type List
#
NAME_LIST ospfAuType { TITLE = "Authentication type", OUT_OF_RANGE = "unknown" } =
	"None",				# 0
	"Simple Password";		# 1



#
# LSA types list
#
NAME_LIST ospfLStype { TITLE = "LS type", OUT_OF_RANGE = "unknown" } =
	"illegal",			# 0
	"Router Links",			# 1
	"Network Links",		# 2
	"IP Network Summary Link",	# 3
	"ASBR Summary Link",		# 4
	"AS External Link",		# 5
	"Group Membership";		# 6



#
# Link types list
#
NAME_LIST ospfLinkType { TITLE = "  Interface type", OUT_OF_RANGE = "unknown" } =
	"illegal",			# 0
	"Point to Point",		# 1
	"Transit Network",		# 2
	"Stub Network",			# 3
	"Virtual Link";			# 4


#
# Vertex Types (for group membership LSAs)
#
NAME_LIST ospfVertexType { TITLE = "Vertex type", OUT_OF_RANGE = "unknown" } =
	"illegal",			# 0
	"Router",			# 1
	"Transit Network";		# 2



#
# Option Flags
#
FLAG_TABLE ospfOptionFlags { TITLE = "Options", OUT_OF_RANGE = "?" } =
	0x01 = "T",
	0x02 = "E",
	0x04 = "MC"
	;



#
# IMS Flags
#
FLAG_TABLE ospfIMSFlags { TITLE = "IMS Flags", OUT_OF_RANGE = "?" } =
	0x01 = "Master",
	0x02 = "More",
	0x04 = "Initial"
	;



#
# VEB Flags
#
FLAG_TABLE ospfVEBFlags { TITLE = "VEB FLags", OUT_OF_RANGE = "?" } =
	0x1 = "B",
	0x2 = "E",
	0x4 = "V",
	0x8 = "W"
	;


DECLARE PST pst_ip_protocols;
DECLARE PST pst_ospf_lsa_list, pst_ospf_link_list, pst_ospf_hello_nl, pst_ospf_router_list;
DECLARE DECODE d_ospf_lsa;



#
# pst_ospf_router_list
# Decode a router list inside a network lsa.  Return to ... when done.
# $10 contains the number of router addresses remaining.
#
DECODE d_ospf_router_list(0) {
	NEXT = pst_ospf_router_list,
	CONSUME = 4,
	PLEVEL = HL_OSPF
	} =

		ip_address( long(0), TRANSLATE, " " ),
		$10 = $10 - 1	# decrement link counter
		;


PST pst_ospf_router_list( $10 ) {
	DEFAULT = d_ospf_router_list
	} = 
		d_ospf_lsa
		;



#
# pst_ospf_link_list
# Decode a links list inside a router lsa.  Return to ... when done.
# $10 contains the number of links remaining.
#
#	NOTE: currently doesn't decode any non-tos-0 metrics.
#
DECODE d_ospf_link_list(0) {
	NEXT = pst_ospf_link_list,
	CONSUME = $1,
	PLEVEL = HL_OSPF
	} =

		nl(0),
		$1 = byte( OSPF_RTR_ntos ),
		$1 = ($1 * 4) + OSPF_RTR_sizeof,
		list( byte(OSPF_RTR_type), 0, ospfLinkType ),
		ip_address( long( OSPF_RTR_id ), DONT_TRANSLATE, "  Link ID" ),
		ip_address( long( OSPF_RTR_data ), DONT_TRANSLATE, "  Link data" ),
		decimal( word( OSPF_RTR_metric0 ), 0, "  Link metric (TOS 0)" ),
		$10 = $10 - 1	# decrement link counter
		;

PST pst_ospf_link_list( $10 ) {
	DEFAULT = d_ospf_link_list
	} = 
		d_ospf_lsa
		;


#
# d_ospf_group_lsa
# Decode a Group Membership LSA
#
DECODE d_ospf_group_lsa(6) {
	CONSUME = $1,
	PLEVEL = HL_OSPF
	} =

		nl(0),
		$1 = word( OSPF_LSH_length ),
		list( long( OSPF_GM_vertex_type ), 0, ospfVertexType ),
		ip_address( long( OSPF_GM_vertex_id ), DONT_TRANSLATE, "Vertex ID" )
		;



#
# d_ospf_external_lsa
# Decode an External LSA.
#
DECODE d_ospf_external_lsa(5) {
	NEXT = d_ospf_lsa,
	CONSUME = $1,
	PLEVEL = HL_OSPF
	} =

		nl(0),
		$1 = word( OSPF_LSH_length ),
		$3 = bitfield_long(1,31,OSPF_EXT_tos_metric),	# Type
		$3 = $3 + 1,
		ip_address( long( OSPF_EXT_mask ), DONT_TRANSLATE, "Network mask" ), 
		print( "  Type, TOS, Metric", "%d, %d, %d", 
			$3,
			bitfield_long(7,24,OSPF_EXT_tos_metric),
			bitfield_long(24,0,OSPF_EXT_tos_metric)),
		ip_address( long( OSPF_EXT_fwd_addr ), TRANSLATE, "  Forwarding address" ), 
		hex( long( OSPF_EXT_ext_tag ), 0, "  External route tag" )
		;



#
# d_ospf_asummary_lsa
# Decode a ASBR Summary LSA.
#
DECODE d_ospf_asummary_lsa(4) {
	NEXT = d_ospf_lsa,
	CONSUME = $1,
	PLEVEL = HL_OSPF
	} =

		$1 = word( OSPF_LSH_length ),
		ip_address( long( OSPF_SUM_mask ), DONT_TRANSLATE, "Network mask" ), 
		decimal( long( OSPF_SUM_tos_metric ), 0, "  Metric (TOS 0)" )
		;



#
# d_ospf_nsummary_lsa
# Decode a Network Summary LSA.
#
DECODE d_ospf_nsummary_lsa(3) {
	NEXT = d_ospf_lsa,
	CONSUME = $1,
	PLEVEL = HL_OSPF
	} = REF d_ospf_asummary_lsa;




#
# d_ospf_network_lsa
# Decode a Network LSA.
#
DECODE d_ospf_network_lsa(2) {
	NEXT = pst_ospf_router_list,
	CONSUME = OSPF_NET_sizeof,
	PLEVEL = HL_OSPF
	} =

		$1 = word( OSPF_LSH_length ),
		$10 = ($1 - OSPF_NET_sizeof) / 4,
		ip_address( long( OSPF_NET_mask ), DONT_TRANSLATE, "Network mask" ), 
		decimal( $10, 0, "Network routers ..." ) { CONDITION = $10 }
		;



#
# d_ospf_router_lsa
# Decode a Router LSA.
#
DECODE d_ospf_router_lsa(1) {
	NEXT = pst_ospf_link_list,
	CONSUME = 4 + OSPF_LSH_sizeof,
	PLEVEL = HL_OSPF
	} =

		$10 = word( OSPF_RTR_links ),
		flags( byte( OSPF_RTR_veb ), 0, ospfVEBFlags ),
		decimal( $10, 0, "Router links ..." ) { CONDITION = $10 }
		;




#
# OSPF LSA Pst
# Switch on the LSA type and decode its contents.  The current offset
# is pointing to the LSA header.
#
PST pst_ospf_lsa_list( byte(OSPF_LSH_type) ) =

		d_ospf_router_lsa,
		d_ospf_network_lsa,
		d_ospf_nsummary_lsa,
		d_ospf_asummary_lsa,
		d_ospf_external_lsa,
		d_ospf_group_lsa
		;



#
# Neighbor list (in HELLO Packets)
# This PST is executed repeatedly until the rest of the packet is emptied.  The
# $1 variable should be initially set to 0 so that only the first line contains
# the neighbor label.
#
DECODE d_ospf_hello_nl1(1) {
	CONSUME = 4, 
	NEXT = pst_ospf_hello_nl,
	PLEVEL = HL_OSPF
	} = 

		ip_address(long(0), TRANSLATE, " " );


DECODE d_ospf_hello_nl(0) {
	NEXT = pst_ospf_hello_nl,
	PLEVEL = HL_OSPF
	} = 

		$1 = remaining(),
		$1 = $1 / 4,
		decimal( $1, 0, "Neighbors ..." ),
		$1 = 1;

PST pst_ospf_hello_nl($1) = d_ospf_hello_nl, d_ospf_hello_nl1;




#
# LSA Request List.
# There are a list of these in an LSREQ packet.
#
DECODE d_ospf_lsreq_list(0) {
	CONSUME = OSPF_LSREQ_sizeof,
	NEXT = d_ospf_lsreq_list,
	PLEVEL = HL_OSPF
	} =
		
		nl(0),
		list( long(OSPF_LSREQ_type), 0, ospfLStype ),
		ip_address( long(OSPF_LSREQ_id), DONT_TRANSLATE, "Link state ID" ),
		ip_address( long(OSPF_LSREQ_adv_router), TRANSLATE, "Advertising router" )
		;



#
# LSA Header
#
DECODE d_ospf_lsh(0) {
	CONSUME = OSPF_LSH_sizeof,
	NEXT = d_ospf_lsh,
	PLEVEL = HL_OSPF
	} =
		
		nl(0),
		list( byte(OSPF_LSH_type), 0, ospfLStype ),
		ip_address( long(OSPF_LSH_id), DONT_TRANSLATE, "Link state ID" ),
		ip_address( long(OSPF_LSH_adv_router), TRANSLATE, "Advertising router" ),
		hex( long(OSPF_LSH_sequence_no), 0, "Sequence number" ),
		flags( byte(OSPF_LSH_options), 0, ospfOptionFlags ),
		hex( word( OSPF_LSH_checksum ), 04, "Checksum" ),
		print( "Age, Checksum, Length", "%d, 0x%04X, %d",
			word( OSPF_LSH_age ),
			word( OSPF_LSH_length ))
		;



#
# LSA Headers List
# List of LSA headers, the number of which is determined by the size of the
# packet.  The current offset must point to the start of the list.
#
DECODE d_ospf_lsh_list(0) {
	NEXT = d_ospf_lsh,
	PLEVEL = HL_OSPF
	} =

		$10 = remaining(),			# Size of headers list
		$10 = $10 / OSPF_LSH_sizeof,		# Number of headers
		decimal( $10, 0, "LSA Headers ..." )
		;



#
# LSA List
# List of LSAs (in Link State Update packets).  First output the LSA header
# (this is identical to d_ospf_lsh except that the checksum field is
# verified), then use the pst_ospf_lsa PST to branch to LSA type
# specific decoding.  The current offset is the start of the list.
#
DECODE d_ospf_lsa(0) {
	NEXT = pst_ospf_lsa_list,
	PLEVEL = HL_OSPF
	} = 
		nl(0),
		list( byte(OSPF_LSH_type), 0, ospfLStype ),
		ip_address( long(OSPF_LSH_id), DONT_TRANSLATE, "Link state ID" ),
		ip_address( long(OSPF_LSH_adv_router), TRANSLATE, "Advertising router" ),
		hex( long(OSPF_LSH_sequence_no), 0, "Sequence number" ),
		flags( byte(OSPF_LSH_options), 0, ospfOptionFlags ),

		# The checksum does not include the LS_AGE field.
		#
		fletcher_checksum( OSPF_LSH_checksum,
				   ((word(OSPF_LSH_length) - 2) << 16) + (OSPF_LSH_checksum - 2),
				   "Checksum (verified)"),

		print( "Age, Length", "%d, %d",
			word( OSPF_LSH_age ),
			word( OSPF_LSH_length ))
		;




#
# d_ospf_lsack
# OSPF LSA Acknowledgement packet.  This is simply a list of LSA headers.
# The current offset is at the start of the OSPF header.
#
DECODE d_ospf_lsack(5) {
	TITLE = "LINK STATE ACK",
	BRIEF_TITLE = "Acknowledgement",
	CONSUME = OSPF_sizeof,
	NEXT = d_ospf_lsh_list,
	PLEVEL = HL_OSPF
	} = ;




#
# d_ospf_lsup
# OSPF LSA Update packet.  Contains a list of complete LSAs.
# The current offset is at the start of the OSPF header.
#
DECODE d_ospf_lsup(4) {
	TITLE = "LINK STATE UPDATE",
	BRIEF_TITLE = "Update",
	CONSUME = OSPF_LSUP_sizeof,
	NEXT = d_ospf_lsa,
	PLEVEL = HL_OSPF
	} = 

		decimal( long(OSPF_LSUP_number), 0, "Update LSAs ...")
		;




#
# d_ospf_lsreq
# OSPF LSA Request packet.
# The current offset is at the start of the OSPF header.
#
DECODE d_ospf_lsreq(3) {
	TITLE = "LINK STATE REQUEST",
	BRIEF_TITLE = "Request",
	CONSUME = OSPF_sizeof,
	NEXT = d_ospf_lsreq_list,
	PLEVEL = HL_OSPF
	} =

		$10 = remaining(),				# Size of list
		$10 = ($10 - OSPF_sizeof) / OSPF_LSREQ_sizeof,	# Number of LSA records
		decimal( $10, 0, "Requested LSAs ..." )
		;




#
# d_ospf_dd
# OSPF Database Description packets.  Current offset is at OSPF header.
#
DECODE d_ospf_dd(2) {
	TITLE = "DATABASE DESCRIPTION",
	BRIEF_TITLE = "Database",
	CONSUME = OSPF_DD_sizeof,
	NEXT = d_ospf_lsh_list,
	PLEVEL = HL_OSPF
	} =

		flags( byte(OSPF_DD_options), 0, ospfOptionFlags ),
		flags( byte(OSPF_DD_ims), 0, ospfIMSFlags ),
		hex( long(OSPF_DD_sequence_no), 0, "DD Sequence number" )
		;



#
# d_ospf_hello
# OSPF Hello Packet.  Current offset is at OSPF header.
#
DECODE d_ospf_hello(1) {
	TITLE = "HELLO",
	BRIEF_TITLE = "Hello",
	CONSUME = OSPF_HELLO_sizeof,
	NEXT = pst_ospf_hello_nl,
	PLEVEL = HL_OSPF
	} =

		flags( byte(OSPF_HELLO_options), 0, ospfOptionFlags ),
		ip_address( long(OSPF_HELLO_designated_router), TRANSLATE, "Designated router" ),
		ip_address( long(OSPF_HELLO_backup_router), TRANSLATE, "Backup router" ),
		ip_address( long(OSPF_HELLO_network_mask), DONT_TRANSLATE, "Network mask" ),
		decimal( byte(OSPF_HELLO_router_priority), 0, "Router priority" ),
		time( word(OSPF_HELLO_hello_int), SECONDS, "Hello interval" ),
		time( long(OSPF_HELLO_dead_interval), SECONDS, "Dead interval" ),
		$1 = 0
		;




#
# PST pst_ospf_types
# Switch on the OSPF packet type.
#
PST pst_ospf_types( byte(OSPF_type) ) {
	TITLE = "OSPF type",
	TYPE = "OSPF"
	} =

		d_ospf_hello,
		d_ospf_dd,
		d_ospf_lsreq,
		d_ospf_lsup,
		d_ospf_lsack
		;




#
# DECODE: d_ospf
# Entry point for OSPF decode.  Currently this just outputs the OSPF header and
# then uses internal DTF routines to decode the rest of the OSPF packet.
#
# CO = IP V4 header.
# $32 = IP header length (in bytes)
#
DECODE d_ospf(89) {
	TITLE = "OSPF",
	ATTACH = pst_ip_protocols,
	NEXT = pst_ospf_types,
	CONSUME = $32,
	PLEVEL = HL_OSPF
	} =

		print( "Version", "V%d", byte( IP_sizeof + OSPF_version ) ),
		length( word( IP_sizeof + OSPF_packet_length ), IP_sizeof, "Packet Length" ),
		ip_address(long(IP_sizeof + OSPF_router_id), DONT_TRANSLATE, "Router ID" ){BRIEF},
		ip_address( long( IP_sizeof + OSPF_area_id ), DONT_TRANSLATE, "Area ID" ),
		hex( word( IP_sizeof + OSPF_checksum ), 4, "Unverified Checksum" ),
		list( word( IP_sizeof + OSPF_autype ), 0, ospfAuType ),
		print( "Authentication", "0x%02X%02X%02X%02X%02X%02X%02X%02X",
			byte( IP_sizeof + OSPF_authentication + 0),
			byte( IP_sizeof + OSPF_authentication + 1 ),
			byte( IP_sizeof + OSPF_authentication + 2 ),
			byte( IP_sizeof + OSPF_authentication + 3 ),
			byte( IP_sizeof + OSPF_authentication + 4 ),
			byte( IP_sizeof + OSPF_authentication + 5 ),
			byte( IP_sizeof + OSPF_authentication + 6 ),
			byte( IP_sizeof + OSPF_authentication + 7 ))
		;
